@@ -1,17 +1,29 @@
-/* Copyright (C) 2000 MySQL AB
+/* Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   Without limiting anything contained in the foregoing, this file,
+   which is part of C Driver for MySQL (Connector/C), is also subject to the
+   Universal FOSS Exception, version 1.0, a copy of which can be found at
+   http://oss.oracle.com/licenses/universal-foss-exception.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "mysys_priv.h"
 #include <my_dir.h>
@@ -37,29 +49,33 @@ int my_rename(const char *from, const char *to, myf MyFlags)
       my_errno=EEXIST;
       error= -1;
       if (MyFlags & MY_FAE+MY_WME)
-	my_error(EE_LINK, MYF(ME_BELL+ME_WAITTANG),from,to,my_errno);
+      {
+        char errbuf[MYSYS_STRERROR_SIZE];
+        my_error(EE_LINK, MYF(ME_BELL+ME_WAITTANG), from, to,
+                 my_errno, my_strerror(errbuf, sizeof(errbuf), my_errno));
+      }
       DBUG_RETURN(error);
     }
     my_errno=save_errno;
   }
 #endif
-#if defined(HAVE_RENAME)
 #if defined(__WIN__)
-  /*
-    On windows we can't rename over an existing file:
-    Remove any conflicting files:
-  */
-  (void) my_delete(to, MYF(0));
-#endif
-  if (rename(from,to))
-#else
-  if (link(from, to) || unlink(from))
-#endif
+  if(!MoveFileEx(from, to, MOVEFILE_COPY_ALLOWED|
+                           MOVEFILE_REPLACE_EXISTING))
   {
+    my_osmaperr(GetLastError());
+#else
+  if (rename(from,to))
+  {
+#endif
     my_errno=errno;
     error = -1;
     if (MyFlags & (MY_FAE+MY_WME))
-      my_error(EE_LINK, MYF(ME_BELL+ME_WAITTANG),from,to,my_errno);
+    {
+      char errbuf[MYSYS_STRERROR_SIZE];
+      my_error(EE_LINK, MYF(ME_BELL+ME_WAITTANG), from, to,
+               my_errno, my_strerror(errbuf, sizeof(errbuf), my_errno));
+    }
   }
   else if (MyFlags & MY_SYNC_DIR)
   {
