@@ -1,19 +1,28 @@
 
 
-/* Copyright (c) 2003, 2005, 2006 MySQL AB
+/*
+   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 
 /**
@@ -195,13 +204,13 @@ callback(int result, NdbTransaction* trans, void* aObject)
 
 
 /**
- * Create table "GARAGE"
+ * Create table "api_async"
  */
 void create_table(MYSQL &mysql) 
 {
   while (mysql_query(&mysql, 
 		  "CREATE TABLE"
-		  "  GARAGE"
+		  "  api_async"
 		  "    (REG_NO INT UNSIGNED NOT NULL,"
 		  "     BRAND CHAR(20) NOT NULL,"
 		  "     COLOR CHAR(20) NOT NULL,"
@@ -210,19 +219,18 @@ void create_table(MYSQL &mysql)
   {
     if (mysql_errno(&mysql) != ER_TABLE_EXISTS_ERROR)
       MYSQLERROR(mysql);
-    std::cout << "MySQL Cluster already has example table: GARAGE. "
+    std::cout << "MySQL Cluster already has example table: api_async. "
 	      << "Dropping it..." << std::endl; 
     drop_table(mysql);
-    create_table(mysql);
   }
 }
 
 /**
- * Drop table GARAGE
+ * Drop table api_async
  */
 void drop_table(MYSQL &mysql)
 {
-  if (mysql_query(&mysql, "DROP TABLE GARAGE"))
+  if (mysql_query(&mysql, "DROP TABLE api_async"))
     MYSQLERROR(mysql);
 }
 
@@ -301,7 +309,7 @@ int populate(Ndb * myNdb, int data, async_callback_t * cbData)
 
   NdbOperation*   myNdbOperation;       // For operations
   const NdbDictionary::Dictionary* myDict= myNdb->getDictionary();
-  const NdbDictionary::Table *myTable= myDict->getTable("GARAGE");
+  const NdbDictionary::Table *myTable= myDict->getTable("api_async");
   if (myTable == NULL) 
     APIERROR(myDict->getNdbError());
 
@@ -436,8 +444,8 @@ int main(int argc, char** argv)
 			     0, mysqld_sock, 0) )
       MYSQLERROR(mysql);
 
-    mysql_query(&mysql, "CREATE DATABASE TEST_DB");
-    if (mysql_query(&mysql, "USE TEST_DB") != 0) MYSQLERROR(mysql);
+    mysql_query(&mysql, "CREATE DATABASE ndb_examples");
+    if (mysql_query(&mysql, "USE ndb_examples") != 0) MYSQLERROR(mysql);
 
     create_table(mysql);
   }
@@ -459,7 +467,7 @@ int main(int argc, char** argv)
   }
 
   Ndb* myNdb = new Ndb( &cluster_connection,
-			"TEST_DB" );  // Object representing the database
+			"ndb_examples" );  // Object representing the database
   if (myNdb->init(1024) == -1) {      // Set max 1024 parallel transactions
     APIERROR(myNdb->getNdbError());
   }
@@ -486,6 +494,4 @@ int main(int argc, char** argv)
   }
   std::cout << "Number of temporary errors: " << tempErrors << std::endl;
   delete myNdb; 
-
-  drop_table(mysql);
 }
