@@ -1,18 +1,26 @@
-/* Copyright (c) 2003, 2005-2007 MySQL AB
-   Use is subject to license terms
+/*
+   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 //
 //  ndbapi_async1.cpp: Using asynchronous transactions in NDB API
@@ -24,6 +32,7 @@
 //  Successful insert.
 
 #include <mysql.h>
+#include <mysqld_error.h>
 #include <NdbApi.hpp>
 
 // Used for cout
@@ -95,12 +104,12 @@ int main(int argc, char** argv)
   /********************************************
    * Connect to database via mysql-c          *
    ********************************************/
-  mysql_query(&mysql, "CREATE DATABASE TEST_DB_1");
-  if (mysql_query(&mysql, "USE TEST_DB_1") != 0) MYSQLERROR(mysql);
+  mysql_query(&mysql, "CREATE DATABASE ndb_examples");
+  if (mysql_query(&mysql, "USE ndb_examples") != 0) MYSQLERROR(mysql);
   create_table(mysql);
 
   Ndb* myNdb = new Ndb( cluster_connection,
-			"TEST_DB_1" );  // Object representing the database
+			"ndb_examples" );  // Object representing the database
 
   NdbTransaction*  myNdbTransaction[2];   // For transactions
   NdbOperation*   myNdbOperation;       // For operations
@@ -114,7 +123,7 @@ int main(int argc, char** argv)
    * Insert (we do two insert transactions in parallel) *
    ******************************************************/
   const NdbDictionary::Dictionary* myDict= myNdb->getDictionary();
-  const NdbDictionary::Table *myTable= myDict->getTable("MYTABLENAME");
+  const NdbDictionary::Table *myTable= myDict->getTable("api_async1");
   if (myTable == NULL)
     APIERROR(myDict->getNdbError());
   for (int i = 0; i < 2; i++) {
@@ -146,34 +155,39 @@ int main(int argc, char** argv)
   delete myNdb;
   delete cluster_connection;
 
-  drop_table(mysql);
-
   ndb_end(0);
   return 0;
 }
 
 /*********************************************************
- * Create a table named MYTABLENAME if it does not exist *
+ * Create a table named api_async1 if it does not exist *
  *********************************************************/
 static void create_table(MYSQL &mysql)
 {
-  if (mysql_query(&mysql, 
-		  "CREATE TABLE"
-		  "  MYTABLENAME"
+  while(mysql_query(&mysql, 
+		  "CREATE TABLE api_async1"
 		  "    (ATTR1 INT UNSIGNED NOT NULL PRIMARY KEY,"
 		  "     ATTR2 INT UNSIGNED NOT NULL)"
 		  "  ENGINE=NDB"))
-    MYSQLERROR(mysql);
+  {
+      if (mysql_errno(&mysql) == ER_TABLE_EXISTS_ERROR)
+      {
+          std::cout << "MySQL Cluster already has example table: api_scan. "
+          << "Dropping it..." << std::endl; 
+          drop_table(mysql);
+      }
+      else MYSQLERROR(mysql);
+  }
 }
 
 /***********************************
- * Drop a table named MYTABLENAME 
+ * Drop a table named api_async1 
  ***********************************/
 static void drop_table(MYSQL &mysql)
 {
   if (mysql_query(&mysql, 
 		  "DROP TABLE"
-		  "  MYTABLENAME"))
+		  "  api_async1"))
     MYSQLERROR(mysql);
 }
 
