@@ -1,17 +1,26 @@
-/* Copyright (C) 2003 MySQL AB
+/*
+   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include "Ndbcntr.hpp"
 
@@ -36,8 +45,8 @@ Ndbcntr::g_sysTable_SYSTAB_0 = {
   "sys/def/SYSTAB_0",
   arrayLength(column_SYSTAB_0), column_SYSTAB_0,
   DictTabInfo::SystemTable,
-  DictTabInfo::AllNodesSmallTable,
-  true, ~0
+  DictTabInfo::HashMapPartition,
+  true, ~0, ~0
 };
 
 // NDB$EVENTS_0
@@ -65,7 +74,7 @@ column_NDBEVENTS_0[] = {
     false, false
   },
   { 5, "ATTRIBUTE_MASK",
-    DictTabInfo::ExtUnsigned, MAXNROFATTRIBUTESINWORDS,
+    DictTabInfo::ExtUnsigned, MAXNROFATTRIBUTESINWORDS_OLD,
     false, false
   },
   { 6, "SUBID",
@@ -76,15 +85,19 @@ column_NDBEVENTS_0[] = {
     DictTabInfo::ExtUnsigned, 1,
     false, false
   },
+  { 8, "ATTRIBUTE_MASK2",
+    DictTabInfo::ExtLongvarbinary, MAX_ATTRIBUTES_IN_TABLE_NDB_EVENTS_0 / 8,
+    false, true
+  }
 };
 
-const Ndbcntr::SysTable
+Ndbcntr::SysTable
 Ndbcntr::g_sysTable_NDBEVENTS_0 = {
   "sys/def/NDB$EVENTS_0",
   arrayLength(column_NDBEVENTS_0), column_NDBEVENTS_0,
   DictTabInfo::SystemTable,
-  DictTabInfo::AllNodesSmallTable,
-  true, ~0
+  DictTabInfo::HashMapPartition,
+  true, ~0, ~0
 };
 
 // all
