@@ -1,18 +1,27 @@
-/* Copyright (c) 2003, 2005, 2006 MySQL AB
-   Use is subject to license terms
+/*
+   Copyright (C) 2005-2008 MySQL AB, 2010 Sun Microsystems, Inc.
+    All rights reserved. Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef Restore_H
 #define Restore_H
@@ -29,8 +38,10 @@
 
 class Restore : public SimulatedBlock
 {
+  friend class RestoreProxy;
+
 public:
-  Restore(Block_context& ctx);
+  Restore(Block_context& ctx, Uint32 instanceNumber = 0);
   virtual ~Restore();
   BLOCK_DEFINES(Restore);
   
@@ -85,6 +96,7 @@ private:
     Uint32 m_fd;          // File pointer
     Uint32 m_file_type;   // File type
     Uint32 m_status;
+    Uint32 m_lcp_version;
 
     enum StatusFlags 
     {
@@ -142,6 +154,8 @@ private:
 
   void parse_error(Signal*, FilePtr, Uint32 line, Uint32 extra);
   int check_file_version(Signal*, Uint32 file_version);
+  void restore_lcp_conf(Signal* signal, FilePtr);
+  void crash_during_restore(FilePtr, Uint32 line, Uint32 errCode);
 public:
   
 private:
