@@ -1,31 +1,43 @@
-/* Copyright (c) 2003, 2006-2008 MySQL AB
-   Use is subject to license terms
+/*
+   Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 
 
 #include "ndbd_malloc_impl.hpp"
 #include <ndb_global.h>
 #include <EventLogger.hpp>
+#include <portlib/NdbMem.h>
 
-#ifndef UNIT_TEST
-extern EventLogger g_eventLogger;
-#else
-extern EventLogger g_eventLogger;
+#ifdef NDB_WIN
+void *sbrk(int increment)
+{
+  return (void*)-1;
+}
 #endif
 
+extern EventLogger * g_eventLogger;
+
 static int f_method_idx = 0;
 #ifdef NDBD_MALLOC_METHOD_SBRK
 static const char * f_method = "SMsm";
@@ -34,6 +46,27 @@ static const char * f_method = "MSms";
 #endif
 #define MAX_CHUNKS 10
 
+#ifdef VM_TRACE
+#ifndef NDBD_RANDOM_START_PAGE
+#define NDBD_RANDOM_START_PAGE
+#endif
+#endif
+
+#ifdef NDBD_RANDOM_START_PAGE
+static Uint32 g_random_start_page_id = 0;
+#endif
+
+/*
+ * For muti-threaded ndbd, these calls are used for locking around
+ * memory allocation operations.
+ *
+ * For single-threaded ndbd, they are no-ops (but still called, to avoid
+ * having to compile this file twice).
+ */
+extern void mt_mem_manager_init();
+extern void mt_mem_manager_lock();
+extern void mt_mem_manager_unlock();
+
 #define ZONE_LO 0
 #define ZONE_HI 1
 
@@ -42,24 +75,25 @@ static const char * f_method = "MSms";
  */
 #define ZONE_LO_BOUND (1u << 19)
 
-struct InitChunk
-{
-  Uint32 m_cnt;
-  Uint32 m_start;
-  Alloc_page* m_ptr;
-};
-
 #include <NdbOut.hpp>
 
+extern void ndbd_alloc_touch_mem(void * p, size_t sz, volatile Uint32 * watchCounter);
+
 static
 bool
-do_malloc(Uint32 pages, InitChunk* chunk)
+do_malloc(Uint32 pages,
+          InitChunk* chunk,
+          Uint32 *watchCounter,
+          void * baseaddress)
 {
   pages += 1;
   void * ptr = 0;
   Uint32 sz = pages;
 
 retry:
+  if (watchCounter)
+    *watchCounter = 9;
+
   char method = f_method[f_method_idx];
   switch(method){
   case 0:
@@ -70,6 +104,9 @@ retry:
     ptr = 0;
     while (ptr == 0)
     {
+      if (watchCounter)
+        *watchCounter = 9;
+
       ptr = sbrk(sizeof(Alloc_page) * sz);
       
       if (ptr == (void*)-1)
@@ -89,6 +126,16 @@ retry:
 	  goto retry;
 	}
       }
+      else if (UintPtr(ptr) < UintPtr(baseaddress))
+      {
+        /**
+         * Unusable memory :(
+         */
+        ndbout_c("sbrk(%lluMb) => %p which is less than baseaddress!!",
+                 Uint64((sizeof(Alloc_page) * sz) >> 20), ptr);
+        f_method_idx++;
+        goto retry;
+      }
     }
     break;
   }
@@ -98,7 +145,18 @@ retry:
     ptr = 0;
     while (ptr == 0)
     {
+      if (watchCounter)
+        *watchCounter = 9;
+
       ptr = malloc(sizeof(Alloc_page) * sz);
+      if (UintPtr(ptr) < UintPtr(baseaddress))
+      {
+        ndbout_c("malloc(%lluMb) => %p which is less than baseaddress!!",
+                 Uint64((sizeof(Alloc_page) * sz) >> 20), ptr);
+        free(ptr);
+        ptr = 0;
+      }
+
       if (ptr == 0)
       {
 	if (method == 'M')
@@ -156,6 +214,8 @@ retry:
 Uint32
 Ndbd_mem_manager::ndb_log2(Uint32 input)
 {
+  if (input > 65535)
+    return 16;
   input = input | (input >> 8);
   input = input | (input >> 4);
   input = input | (input >> 2);
@@ -170,22 +230,42 @@ Ndbd_mem_manager::ndb_log2(Uint32 input)
 Ndbd_mem_manager::Ndbd_mem_manager()
 {
   m_base_page = 0;
-  bzero(m_buddy_lists, sizeof(m_buddy_lists));
-  bzero(m_resource_limit, sizeof(m_resource_limit));
+  memset(m_buddy_lists, 0, sizeof(m_buddy_lists));
+  memset(m_resource_limit, 0, sizeof(m_resource_limit));
   
   if (sizeof(Free_page_data) != (4 * (1 << FPD_2LOG)))
   {
-    g_eventLogger.error("Invalid build, ndbd_malloc_impl.cpp:%d", __LINE__);
+    g_eventLogger->error("Invalid build, ndbd_malloc_impl.cpp:%d", __LINE__);
     abort();
   }
+  mt_mem_manager_init();
+}
+
+void*
+Ndbd_mem_manager::get_memroot() const
+{
+#ifdef NDBD_RANDOM_START_PAGE
+  return (void*)(m_base_page - g_random_start_page_id);
+#else
+  return (void*)m_base_page;
+#endif
 }
 
 /**
+ *
+ * resource 0 has following semantics:
+ *
+ * m_min  - remaining reserved for other resources
+ * m_curr - sum(m_curr) for other resources (i.e total in use)
+ * m_max  - totally allocated from OS
+ *
+ * resource N has following semantics:
+ *
  * m_min = reserved
- * m_curr = current
+ * m_curr = currently used
  * m_max = max alloc, 0 = no limit
+ *
  */
-
 void
 Ndbd_mem_manager::set_resource_limit(const Resource_limit& rl)
 {
@@ -193,11 +273,13 @@ Ndbd_mem_manager::set_resource_limit(const Resource_limit& rl)
   assert(id < XX_RL_COUNT);
   
   Uint32 reserve = id ? rl.m_min : 0;
+  mt_mem_manager_lock();
   Uint32 current_reserved = m_resource_limit[0].m_min;
   
   m_resource_limit[id] = rl;
   m_resource_limit[id].m_curr = 0;
   m_resource_limit[0].m_min = current_reserved + reserve;
+  mt_mem_manager_unlock();
 }
 
 bool
@@ -205,7 +287,9 @@ Ndbd_mem_manager::get_resource_limit(Uint32 id, Resource_limit& rl) const
 {
   if (id < XX_RL_COUNT)
   {
+    mt_mem_manager_lock();
     rl = m_resource_limit[id];
+    mt_mem_manager_unlock();
     return true;
   }
   return false;
@@ -244,12 +328,27 @@ check_resource_limits(Resource_limit* rl)
 #endif
 }
 
+int
+cmp_chunk(const void * chunk_vptr_1, const void * chunk_vptr_2)
+{
+  InitChunk * ptr1 = (InitChunk*)chunk_vptr_1;
+  InitChunk * ptr2 = (InitChunk*)chunk_vptr_2;
+  if (ptr1->m_ptr < ptr2->m_ptr)
+    return -1;
+  if (ptr1->m_ptr > ptr2->m_ptr)
+    return 1;
+  assert(false);
+  return 0;
+}
 
 bool
-Ndbd_mem_manager::init(bool alloc_less_memory)
+Ndbd_mem_manager::init(Uint32 *watchCounter, bool alloc_less_memory)
 {
   assert(m_base_page == 0);
 
+  if (watchCounter)
+    *watchCounter = 9;
+
   Uint32 pages = 0;
   Uint32 max_page = 0;
   Uint32 reserved = m_resource_limit[0].m_min;
@@ -267,48 +366,64 @@ Ndbd_mem_manager::init(bool alloc_less_memory)
     m_resource_limit[0].m_min = pages;
   }
   
-  g_eventLogger.info("Ndbd_mem_manager::init(%d) min: %dMb initial: %dMb",
-		     alloc_less_memory,
-		     (sizeof(Alloc_page)*m_resource_limit[0].m_min)>>20,
-		     (sizeof(Alloc_page)*pages)>>20);
-  
+  const Uint64 pg = Uint64(sizeof(Alloc_page));
+  g_eventLogger->info("Ndbd_mem_manager::init(%d) min: %lluMb initial: %lluMb",
+                      alloc_less_memory,
+                      (pg*m_resource_limit[0].m_min)>>20,
+                      (pg*pages) >> 20);
+
   if (pages == 0)
   {
-    return 0;
+    return false;
   }
 
+#if SIZEOF_CHARP == 4
+  Uint64 sum = (pg*pages); 
+  if (sum >= (Uint64(1) << 32))
+  {
+    g_eventLogger->error("Trying to allocate more that 4Gb with 32-bit binary!!");
+    return false;
+  }
+#endif
+
+#ifdef NDBD_RANDOM_START_PAGE
+  /**
+   * In order to find bad-users of page-id's
+   *   we add a random offset to the page-id's returned
+   *   however, due to ZONE_LO that offset can't be that big
+   *   (since we at get_page don't know if it's a HI/LO page)
+   */
+  Uint32 max_rand_start = ZONE_LO_BOUND - 1;
+  if (max_rand_start > pages)
+  {
+    max_rand_start -= pages;
+    if (max_rand_start > 0x10000)
+      g_random_start_page_id = 0x10000 + (rand() % (max_rand_start - 0x10000));
+    else if (max_rand_start)
+      g_random_start_page_id = rand() % max_rand_start;
+
+    assert(Uint64(pages) + Uint64(g_random_start_page_id) <= 0xFFFFFFFF);
+
+    ndbout_c("using g_random_start_page_id: %u (%.8x)",
+             g_random_start_page_id, g_random_start_page_id);
+  }
+#endif
+
   /**
    * Do malloc
    */
-
-  Uint32 cnt = 0;
-  struct InitChunk chunks[MAX_CHUNKS];
-  bzero(chunks, sizeof(chunks));
-
   Uint32 allocated = 0;
-  while (cnt < MAX_CHUNKS && allocated < pages)
+  while (m_unmapped_chunks.size() < MAX_CHUNKS && allocated < pages)
   {
-    InitChunk chunk; 
-    LINT_INIT(chunk.m_start);
+    InitChunk chunk;
+    memset(&chunk, 0, sizeof(chunk));
     
-#if defined(_lint) || defined(FORCE_INIT_OF_VARS)
-    memset((char*) &chunk, 0 , sizeof(chunk));
-#endif
-
-    if (do_malloc(pages - allocated, &chunk))
+    if (do_malloc(pages - allocated, &chunk, watchCounter, m_base_page))
     {
-      Uint32 i = 0;
-      for(; i<cnt ; i++)
-      {
-	if (chunk.m_ptr < chunks[i].m_ptr)
-	{
-	  for (Uint32 j = cnt; j > i; j--)
-	    chunks[j] = chunks[j-1];
-	  break;
-	}
-      }
-      cnt++;
-      chunks[i] = chunk;
+      if (watchCounter)
+        *watchCounter = 9;
+
+      m_unmapped_chunks.push_back(chunk);
       allocated += chunk.m_cnt;
     }
     else
@@ -319,35 +434,43 @@ Ndbd_mem_manager::init(bool alloc_less_memory)
   
   if (allocated < m_resource_limit[0].m_min)
   {
-    g_eventLogger.
+    g_eventLogger->
       error("Unable to alloc min memory from OS: min: %lldMb "
-	    " allocated: %lldMb", 
-	    (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_min) >> 20,
-	    (Uint64)(sizeof(Alloc_page)*allocated) >> 20);
+            " allocated: %lldMb",
+            (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_min) >> 20,
+            (Uint64)(sizeof(Alloc_page)*allocated) >> 20);
     return false;
   }
   else if (allocated < pages)
   {
-    g_eventLogger.
+    g_eventLogger->
       warning("Unable to alloc requested memory from OS: min: %lldMb"
-	      " requested: %lldMb allocated: %lldMb",
-	      (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_min)>>20,
-	      (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_max)>>20,
-	      (Uint64)(sizeof(Alloc_page)*allocated)>>20);
+              " requested: %lldMb allocated: %lldMb",
+              (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_min)>>20,
+              (Uint64)(sizeof(Alloc_page)*m_resource_limit[0].m_max)>>20,
+              (Uint64)(sizeof(Alloc_page)*allocated)>>20);
     if (!alloc_less_memory)
       return false;
   }
+
+  /**
+   * Sort chunks...
+   */
+  qsort(m_unmapped_chunks.getBase(), m_unmapped_chunks.size(),
+        sizeof(InitChunk), cmp_chunk);
+
+  m_base_page = m_unmapped_chunks[0].m_ptr;
   
-  m_base_page = chunks[0].m_ptr;
-  
-  for (Uint32 i = 0; i<cnt; i++)
+  for (Uint32 i = 0; i<m_unmapped_chunks.size(); i++)
   {
-    UintPtr start = UintPtr(chunks[i].m_ptr) - UintPtr(m_base_page);
+    UintPtr start = UintPtr(m_unmapped_chunks[i].m_ptr) - UintPtr(m_base_page);
     start >>= (2 + BMW_2LOG);
-    UintPtr last = start + chunks[i].m_cnt;
-    chunks[i].m_start = start;
     assert((Uint64(start) >> 32) == 0);
-    
+    m_unmapped_chunks[i].m_start = Uint32(start);
+    Uint64 last64 = start + m_unmapped_chunks[i].m_cnt;
+    assert((last64 >> 32) == 0);
+    Uint32 last = Uint32(last64);
+
     if (last > max_page)
       max_page = last;
   }
@@ -355,17 +478,101 @@ Ndbd_mem_manager::init(bool alloc_less_memory)
   m_resource_limit[0].m_resource_id = max_page;
   m_resource_limit[0].m_min = reserved;
   m_resource_limit[0].m_max = 0;
-  
-  for (Uint32 i = 0; i<cnt; i++)
-  {
-    grow(chunks[i].m_start, chunks[i].m_cnt);
-  }
-  
-  check_resource_limits(m_resource_limit);
 
   return true;
 }
 
+void
+Ndbd_mem_manager::map(Uint32 * watchCounter, bool memlock, Uint32 resources[])
+{
+  Uint32 limit = ~(Uint32)0;
+  Uint32 sofar = 0;
+
+  if (resources != 0)
+  {
+    limit = 0;
+    for (Uint32 i = 0; resources[i] ; i++)
+    {
+      limit += m_resource_limit[resources[i]].m_min;
+    }
+  }
+
+  while (m_unmapped_chunks.size() && sofar < limit)
+  {
+    Uint32 remain = limit - sofar;
+
+    unsigned idx = m_unmapped_chunks.size() - 1;
+    InitChunk * chunk = &m_unmapped_chunks[idx];
+    if (watchCounter)
+      *watchCounter = 9;
+
+    if (chunk->m_cnt > remain)
+    {
+      /**
+       * Split chunk
+       */
+      Uint32 extra = chunk->m_cnt - remain;
+      chunk->m_cnt = remain;
+
+      InitChunk newchunk;
+      newchunk.m_start = chunk->m_start + remain;
+      newchunk.m_ptr = m_base_page + newchunk.m_start;
+      newchunk.m_cnt = extra;
+      m_unmapped_chunks.push_back(newchunk);
+
+      // pointer could have changed after m_unmapped_chunks.push_back
+      chunk = &m_unmapped_chunks[idx];
+    }
+
+    ndbd_alloc_touch_mem(chunk->m_ptr,
+                         chunk->m_cnt * sizeof(Alloc_page),
+                         watchCounter);
+
+    if (memlock)
+    {
+      /**
+       * memlock pages that I added...
+       */
+      if (watchCounter)
+        *watchCounter = 9;
+
+      /**
+       * Don't memlock everything in one go...
+       *   cause then process won't be killable
+       */
+      const Alloc_page * start = chunk->m_ptr;
+      Uint32 cnt = chunk->m_cnt;
+      while (cnt > 32768) // 1G
+      {
+        if (watchCounter)
+          *watchCounter = 9;
+
+        NdbMem_MemLock(start, 32768 * sizeof(Alloc_page));
+        start += 32768;
+        cnt -= 32768;
+      }
+      if (watchCounter)
+        *watchCounter = 9;
+
+      NdbMem_MemLock(start, cnt * sizeof(Alloc_page));
+    }
+
+    grow(chunk->m_start, chunk->m_cnt);
+    sofar += chunk->m_cnt;
+
+    m_unmapped_chunks.erase(idx);
+  }
+  
+  mt_mem_manager_lock();
+  check_resource_limits(m_resource_limit);
+  mt_mem_manager_unlock();
+
+  if (resources == 0 && memlock)
+  {
+    NdbMem_MemLockAll(1);
+  }
+}
+
 #include <NdbOut.hpp>
 
 void
@@ -403,10 +610,10 @@ Ndbd_mem_manager::grow(Uint32 start, Uint32 cnt)
 	     " - Unable to use due to bitmap pages missaligned!!",
 	     __LINE__, start, cnt, start, (start_bmp << BPP_2LOG),
 	     (cnt >> (20 - 15)));
-    g_eventLogger.error("ndbd_malloc_impl.cpp:%d:grow(%d, %d) not using %uMb"
-			" - Unable to use due to bitmap pages missaligned!!",
-			__LINE__, start, cnt,
-			(cnt >> (20 - 15)));
+    g_eventLogger->error("ndbd_malloc_impl.cpp:%d:grow(%d, %d) not using %uMb"
+                         " - Unable to use due to bitmap pages missaligned!!",
+                         __LINE__, start, cnt,
+                         (cnt >> (20 - 15)));
 
     dump();
     return;
@@ -427,6 +634,7 @@ Ndbd_mem_manager::grow(Uint32 start, Uint32 cnt)
 found:
   if (cnt)
   {
+    mt_mem_manager_lock();
     m_resource_limit[0].m_curr += cnt;
     m_resource_limit[0].m_max += cnt;
     if (start >= ZONE_LO_BOUND)
@@ -454,6 +662,7 @@ found:
       release(start, cnt0);
       release(ZONE_LO_BOUND, cnt1);
     }
+    mt_mem_manager_unlock();
   }
 }
 
@@ -511,7 +720,7 @@ Ndbd_mem_manager::alloc(AllocZone zone,
       return;
     * pages = save;
   }
-  
+
   alloc_impl(ZONE_LO, ret, pages, min);
 }
 
@@ -655,6 +864,7 @@ Ndbd_mem_manager::remove_free_list(Uint32 zone, Uint32 start, Uint32 list)
 void
 Ndbd_mem_manager::dump() const
 {
+  mt_mem_manager_lock();
   for (Uint32 zone = 0; zone < 2; zone ++)
   {
     for (Uint32 i = 0; i<16; i++)
@@ -680,6 +890,7 @@ Ndbd_mem_manager::dump() const
              m_resource_limit[i].m_max);
     }
   }
+  mt_mem_manager_unlock();
 }
 
 void*
@@ -687,6 +898,7 @@ Ndbd_mem_manager::alloc_page(Uint32 type, Uint32* i, AllocZone zone)
 {
   Uint32 idx = type & RG_MASK;
   assert(idx && idx < XX_RL_COUNT);
+  mt_mem_manager_lock();
   Resource_limit tot = m_resource_limit[0];
   Resource_limit rl = m_resource_limit[idx];
 
@@ -707,10 +919,16 @@ Ndbd_mem_manager::alloc_page(Uint32 type, Uint32* i, AllocZone zone)
       m_resource_limit[idx].m_curr = rl.m_curr + cnt;
 
       check_resource_limits(m_resource_limit);
+      mt_mem_manager_unlock();
+#ifdef NDBD_RANDOM_START_PAGE
+      *i += g_random_start_page_id;
+      return m_base_page + *i - g_random_start_page_id;
+#else
       return m_base_page + *i;
+#endif
     }
   }
-
+  mt_mem_manager_unlock();
   return 0;
 }
 
@@ -719,9 +937,14 @@ Ndbd_mem_manager::release_page(Uint32 type, Uint32 i)
 {
   Uint32 idx = type & RG_MASK;
   assert(idx && idx < XX_RL_COUNT);
+  mt_mem_manager_lock();
   Resource_limit tot = m_resource_limit[0];
   Resource_limit rl = m_resource_limit[idx];
-  
+
+#ifdef NDBD_RANDOM_START_PAGE
+  i -= g_random_start_page_id;
+#endif
+
   Uint32 sub = (rl.m_curr <= rl.m_min) ? 1 : 0; // Over min ?
   release(i, 1);
   m_resource_limit[0].m_curr = tot.m_curr - 1;
@@ -729,6 +952,7 @@ Ndbd_mem_manager::release_page(Uint32 type, Uint32 i)
   m_resource_limit[idx].m_curr = rl.m_curr - 1;
 
   check_resource_limits(m_resource_limit);
+  mt_mem_manager_unlock();
 }
 
 void
@@ -736,6 +960,7 @@ Ndbd_mem_manager::alloc_pages(Uint32 type, Uint32* i, Uint32 *cnt, Uint32 min)
 {
   Uint32 idx = type & RG_MASK;
   assert(idx && idx < XX_RL_COUNT);
+  mt_mem_manager_lock();
   Resource_limit tot = m_resource_limit[0];
   Resource_limit rl = m_resource_limit[idx];
 
@@ -774,7 +999,7 @@ Ndbd_mem_manager::alloc_pages(Uint32 type, Uint32* i, Uint32 *cnt, Uint32 min)
   if (likely(req))
   {
     // Hi order allocations can always use any zone
-    alloc(NDB_ZONE_ANY, i, &req, 1); 
+    alloc(NDB_ZONE_ANY, i, &req, min); 
     * cnt = req;
     if (unlikely(req < res0)) // Got min than what was reserved :-(
     {
@@ -787,9 +1012,17 @@ Ndbd_mem_manager::alloc_pages(Uint32 type, Uint32* i, Uint32 *cnt, Uint32 min)
     m_resource_limit[0].m_min = tot.m_min - res0;
     m_resource_limit[idx].m_curr = rl.m_curr + req;
     check_resource_limits(m_resource_limit);
+    mt_mem_manager_unlock();
+#ifdef NDBD_RANDOM_START_PAGE
+    *i += g_random_start_page_id;
+#endif
     return ;
   }
+  mt_mem_manager_unlock();
   * cnt = req;
+#ifdef NDBD_RANDOM_START_PAGE
+  *i += g_random_start_page_id;
+#endif
   return;
 }
 
@@ -798,9 +1031,14 @@ Ndbd_mem_manager::release_pages(Uint32 type, Uint32 i, Uint32 cnt)
 {
   Uint32 idx = type & RG_MASK;
   assert(idx && idx < XX_RL_COUNT);
+  mt_mem_manager_lock();
   Resource_limit tot = m_resource_limit[0];
   Resource_limit rl = m_resource_limit[idx];
-  
+
+#ifdef NDBD_RANDOM_START_PAGE
+  i -= g_random_start_page_id;
+#endif
+
   release(i, cnt);
 
   Uint32 currnew = rl.m_curr - cnt;
@@ -818,6 +1056,7 @@ Ndbd_mem_manager::release_pages(Uint32 type, Uint32 i, Uint32 cnt)
   m_resource_limit[0].m_curr = tot.m_curr - cnt;
   m_resource_limit[idx].m_curr = currnew;
   check_resource_limits(m_resource_limit);
+  mt_mem_manager_unlock();
 }
 
 #ifdef UNIT_TEST
@@ -882,12 +1121,12 @@ main(int argc, char** argv)
   Timer timer[4];
   printf("Startar modul test av Page Manager %dMb %ds\n", 
 	 (sz >> 5), run_time);
-  g_eventLogger.createConsoleHandler();
-  g_eventLogger.setCategory("keso");
-  g_eventLogger.enable(Logger::LL_ON, Logger::LL_INFO);
-  g_eventLogger.enable(Logger::LL_ON, Logger::LL_CRITICAL);
-  g_eventLogger.enable(Logger::LL_ON, Logger::LL_ERROR);
-  g_eventLogger.enable(Logger::LL_ON, Logger::LL_WARNING);
+  g_eventLogger->createConsoleHandler();
+  g_eventLogger->setCategory("keso");
+  g_eventLogger->enable(Logger::LL_ON, Logger::LL_INFO);
+  g_eventLogger->enable(Logger::LL_ON, Logger::LL_CRITICAL);
+  g_eventLogger->enable(Logger::LL_ON, Logger::LL_ERROR);
+  g_eventLogger->enable(Logger::LL_ON, Logger::LL_WARNING);
   
 #define DEBUG 0
 
@@ -903,7 +1142,7 @@ main(int argc, char** argv)
   rl.m_resource_id = 1;
   mem.set_resource_limit(rl);
   
-  mem.init();
+  mem.init(NULL);
   mem.dump();
   printf("pid: %d press enter to continue\n", getpid());
   fgets(buf, sizeof(buf), stdin);
@@ -1013,4 +1252,5 @@ main(int argc, char** argv)
 template class Vector<Chunk>;
 
 #endif
-template class Vector<Uint32>;
+
+template class Vector<InitChunk>;
