@@ -1,17 +1,29 @@
-/* Copyright (C) 2000 MySQL AB
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   Without limiting anything contained in the foregoing, this file,
+   which is part of C Driver for MySQL (Connector/C), is also subject to the
+   Universal FOSS Exception, version 1.0, a copy of which can be found at
+   http://oss.oracle.com/licenses/universal-foss-exception.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <my_global.h>
 #include "m_string.h"
@@ -356,10 +368,10 @@ NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
 };
 
 static
-int my_mb_wc_latin1(CHARSET_INFO *cs  __attribute__((unused)),
+int my_mb_wc_latin1(const CHARSET_INFO *cs  MY_ATTRIBUTE((unused)),
 		    my_wc_t *wc,
 		    const uchar *str,
-		    const uchar *end __attribute__((unused)))
+		    const uchar *end MY_ATTRIBUTE((unused)))
 {
   if (str >= end)
     return MY_CS_TOOSMALL;
@@ -369,17 +381,20 @@ int my_mb_wc_latin1(CHARSET_INFO *cs  __attribute__((unused)),
 }
 
 static
-int my_wc_mb_latin1(CHARSET_INFO *cs  __attribute__((unused)),
+int my_wc_mb_latin1(const CHARSET_INFO *cs  MY_ATTRIBUTE((unused)),
 		    my_wc_t wc,
 		    uchar *str,
-		    uchar *end __attribute__((unused)))
+		    uchar *end MY_ATTRIBUTE((unused)))
 {
   uchar *pl;
   
   if (str >= end)
     return MY_CS_TOOSMALL;
   
-  pl= uni_to_cs[(wc>>8) & 0xFF];
+  if (wc > 0xFFFF)
+    return MY_CS_ILUNI;
+  
+  pl= uni_to_cs[wc >> 8];
   str[0]= pl ? pl[wc & 0xFF] : '\0';
   return (!str[0] && wc) ? MY_CS_ILUNI : 1;
 }
@@ -428,11 +443,10 @@ CHARSET_INFO my_charset_latin1=
     to_lower_latin1,
     to_upper_latin1,
     sort_order_latin1,
-    NULL,		/* contractions */
-    NULL,		/* sort_order_big*/
+    NULL,		/* uca          */
     cs_to_uni,		/* tab_to_uni   */
     NULL,		/* tab_from_uni */
-    my_unicase_default, /* caseinfo     */
+    &my_unicase_default,/* caseinfo     */
     NULL,		/* state_map    */
     NULL,		/* ident_map    */
     1,			/* strxfrm_multiply */
@@ -444,6 +458,8 @@ CHARSET_INFO my_charset_latin1=
     255,		/* max_sort_char */
     ' ',                /* pad char      */
     0,                  /* escape_with_backslash_is_dangerous */
+    1,                  /* levels_for_compare */
+    1,                  /* levels_for_order   */
     &my_charset_handler,
     &my_collation_8bit_simple_ci_handler
 };
@@ -542,7 +558,8 @@ uchar combo2map[]={
 */
 
 
-static int my_strnncoll_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
+static int my_strnncoll_latin1_de(const CHARSET_INFO *cs
+                                  MY_ATTRIBUTE((unused)),
 				  const uchar *a, size_t a_length,
 				  const uchar *b, size_t b_length,
                                   my_bool b_is_prefix)
@@ -583,7 +600,8 @@ static int my_strnncoll_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
 }
 
 
-static int my_strnncollsp_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
+static int my_strnncollsp_latin1_de(const CHARSET_INFO *cs
+                                    MY_ATTRIBUTE((unused)),
 				    const uchar *a, size_t a_length,
 				    const uchar *b, size_t b_length,
                                     my_bool diff_if_only_endspace_difference)
@@ -655,26 +673,29 @@ static int my_strnncollsp_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
 }
 
 
-static size_t my_strnxfrm_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
-                                    uchar *dest, size_t len,
-                                    const uchar *src, size_t srclen)
+static size_t
+my_strnxfrm_latin1_de(const CHARSET_INFO *cs,
+                      uchar *dst, size_t dstlen, uint nweights,
+                      const uchar* src, size_t srclen, uint flags)
 {
-  const uchar *de = dest + len;
-  const uchar *se = src + srclen;
-  for ( ; src < se && dest < de ; src++)
+  uchar *de= dst + dstlen;
+  const uchar *se= src + srclen;
+  uchar *d0= dst;
+  for ( ; src < se && dst < de && nweights; src++, nweights--)
   {
-    uchar chr=combo1map[*src];
-    *dest++=chr;
-    if ((chr=combo2map[*src]) && dest < de)
-      *dest++=chr;
+    uchar chr= combo1map[*src];
+    *dst++= chr;
+    if ((chr= combo2map[*src]) && dst < de && nweights > 1)
+    {
+      *dst++= chr;
+      nweights--;
+    }
   }
-  if (dest < de)
-    bfill(dest, de - dest, ' ');
-  return (int) len;
+  return my_strxfrm_pad_desc_and_reverse(cs, d0, dst, de, nweights, flags, 0);
 }
 
 
-void my_hash_sort_latin1_de(CHARSET_INFO *cs __attribute__((unused)),
+void my_hash_sort_latin1_de(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
 			    const uchar *key, size_t len,
 			    ulong *nr1, ulong *nr2)
 {
@@ -727,11 +748,10 @@ CHARSET_INFO my_charset_latin1_german2_ci=
   to_lower_latin1,
   to_upper_latin1,
   sort_order_latin1_de,
-  NULL,					/* contractions */
-  NULL,					/* sort_order_big*/
+  NULL,					/* uca          */
   cs_to_uni,				/* tab_to_uni   */
   NULL,					/* tab_from_uni */
-  my_unicase_default,                   /* caseinfo     */
+  &my_unicase_default,                  /* caseinfo     */
   NULL,					/* state_map    */
   NULL,					/* ident_map    */
   2,					/* strxfrm_multiply */
@@ -743,6 +763,8 @@ CHARSET_INFO my_charset_latin1_german2_ci=
   247,					/* max_sort_char */
   ' ',                                  /* pad char      */
   0,                                    /* escape_with_backslash_is_dangerous */
+  1,                                    /* levels_for_compare */
+  1,                                    /* levels_for_order   */
   &my_charset_handler,
   &my_collation_german2_ci_handler
 };
@@ -760,11 +782,10 @@ CHARSET_INFO my_charset_latin1_bin=
   to_lower_latin1,
   to_upper_latin1,
   NULL,					/* sort_order   */
-  NULL,					/* contractions */
-  NULL,					/* sort_order_big*/
+  NULL,					/* uca          */
   cs_to_uni,				/* tab_to_uni   */
   NULL,					/* tab_from_uni */
-  my_unicase_default,                   /* caseinfo     */
+  &my_unicase_default,                  /* caseinfo     */
   NULL,					/* state_map    */
   NULL,					/* ident_map    */
   1,					/* strxfrm_multiply */
@@ -776,6 +797,8 @@ CHARSET_INFO my_charset_latin1_bin=
   255,					/* max_sort_char */
   ' ',                                  /* pad char      */
   0,                                    /* escape_with_backslash_is_dangerous */
+  1,                                    /* levels_for_compare */
+  1,                                    /* levels_for_order   */
   &my_charset_handler,
   &my_collation_8bit_bin_handler
 };
